@@ -28,21 +28,21 @@ from zope.location import locate
 from zope.schema.fieldproperty import FieldProperty
 
 from pyams_form.button import Buttons, Handlers, button_and_handler
-from pyams_form.events import DataExtractedEvent, FormCreatedEvent
+from pyams_form.events import DataExtractedEvent, FormCreatedEvent, FormUpdatedEvent
 from pyams_form.field import Fields
 from pyams_form.interfaces import DISPLAY_MODE, INPUT_MODE
 from pyams_form.interfaces.button import IActionErrorEvent, IActions, WidgetActionExecutionError
 from pyams_form.interfaces.error import IErrorViewSnippet
 from pyams_form.interfaces.form import IActionForm, IAddForm, IButtonForm, IDisplayForm, \
-    IEditForm, IFieldsForm, IForm, IFormAware, IGroup, IGroupManager, IHandlerForm, \
-    IInnerSubForm, IInnerTabForm, IInputForm
+    IEditForm, IFieldsForm, IForm, IFormAware, IFormContent, IFormFields, IGroup, IGroupManager, \
+    IHandlerForm, IInnerSubForm, IInnerTabForm, IInputForm
 from pyams_form.interfaces.widget import IWidgets
 from pyams_form.util import changed_field
 from pyams_security.interfaces.base import FORBIDDEN_PERMISSION
 from pyams_security.permission import get_edit_permission
 from pyams_template.template import get_content_template, get_layout_template
 from pyams_utils.adapter import ContextRequestAdapter
-from pyams_utils.factory import get_object_factory, is_interface
+from pyams_utils.factory import create_object
 from pyams_utils.interfaces import ICacheKeyValue
 from pyams_utils.interfaces.form import IDataManager, NOT_CHANGED
 from pyams_utils.url import absolute_url
@@ -90,7 +90,11 @@ def apply_changes(form, content, data):
 
 
 def extends(*args, **kwargs):
-    """Extend fields, buttons and handlers"""
+    """Extend fields, buttons and handlers
+
+    Please note that extending fields only works if parent class is using static
+    fields definition!
+    """
     frame = sys._getframe(1)  # pylint: disable=protected-access
     f_locals = frame.f_locals
     if not kwargs.get('ignore_fields', False):
@@ -146,8 +150,6 @@ def handle_action_error(event):
 class BaseForm(ContextRequestAdapter):
     """A base form."""
 
-    fields = Fields()
-
     title = None
     legend = None
     required_label = _('<span class="required">*</span>&ndash; required')
@@ -167,6 +169,15 @@ class BaseForm(ContextRequestAdapter):
     ignore_readonly = False
     ignore_required_on_extract = False
 
+    @volatile_property
+    def fields(self):
+        """Form fields getter"""
+        fields = self.request.registry.queryMultiAdapter((self.context, self.request, self),
+                                                         IFormFields)
+        if fields is None:
+            fields = Fields()
+        return fields
+
     @property
     def mode(self):
         """Current mode getter"""
@@ -197,9 +208,18 @@ class BaseForm(ContextRequestAdapter):
         content = self.get_content()
         return get_edit_permission(self.request, content, self)
 
+    @reify
+    def form_content(self):
+        """Form content getter"""
+        content = self.request.registry.queryMultiAdapter((self.context, self.request, self),
+                                                          IFormContent)
+        if content is None:
+            content = self.context
+        return content
+
     def get_content(self):
         """See interfaces.form.IForm"""
-        return self.context
+        return self.form_content
 
     @property
     def required_info(self):
@@ -213,19 +233,21 @@ class BaseForm(ContextRequestAdapter):
     def subforms(self):
         """Get list of internal sub-forms"""
         registry = self.request.registry
-        return sorted((adapter
-                       for name, adapter in registry.getAdapters((self.context, self.request, self),
-                                                                 IInnerSubForm)),
-                      key=get_form_weight)
+        return sorted((
+            adapter
+            for name, adapter in registry.getAdapters((self.context, self.request, self),
+                                                      IInnerSubForm)
+        ), key=get_form_weight)
 
     @reify
     def tabforms(self):
         """Get list of internal tab-forms"""
         registry = self.request.registry
-        return sorted((adapter
-                       for name, adapter in registry.getAdapters((self.context, self.request, self),
-                                                                 IInnerTabForm)),
-                      key=get_form_weight)
+        return sorted((
+            adapter
+            for name, adapter in registry.getAdapters((self.context, self.request, self),
+                                                      IInnerTabForm)
+        ), key=get_form_weight)
 
     def get_forms(self, include_self=True):
         """Get all forms associated with this form"""
@@ -236,15 +258,18 @@ class BaseForm(ContextRequestAdapter):
             for group in manager.groups:
                 yield from group.get_forms()
         for form in self.subforms:
-            yield from form.get_forms()
+            if IInnerSubForm.providedBy(form):
+                yield from form.get_forms()
         for form in self.tabforms:
-            yield from form.get_forms()
+            if IInnerTabForm.providedBy(form):
+                yield from form.get_forms()
 
     def update(self):
         """See interfaces.form.IForm"""
         self.update_widgets()
         [subform.update() for subform in self.subforms]  # pylint: disable=expression-not-assigned
         [tabform.update() for tabform in self.tabforms]  # pylint: disable=expression-not-assigned
+        self.request.registry.notify(FormUpdatedEvent(self))
 
     def update_widgets(self, prefix=None):
         """See interfaces.form.IForm"""
@@ -429,9 +454,7 @@ class AddForm(Form):
     def create(self, data):  # pylint: disable=unused-argument
         """Create new content from form data"""
         if self.content_factory is not None:
-            factory = get_object_factory(self.content_factory) \
-                if is_interface(self.content_factory) else self.content_factory
-            return factory()  # pylint: disable=not-callable
+            return create_object(self.content_factory)
         raise NotImplementedError
 
     def add(self, obj):  # pylint: disable=redefined-builtin
